/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_82519226_024B_4F87_A849_B05CEB2F0A28
#define INCLUDE_GUARD_82519226_024B_4F87_A849_B05CEB2F0A28

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ContainerApi.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "Api/UrlApi.h"


//////////////////////////////////////////////////////////////////////////////////////////////////

struct ContainerCallbackApi;


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ContainerApi
//
//  Every DLL housing one or more container translation modules should support this interface
//  via Media_GetApi().
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct ContainerApi: TivoApi::Api
{

    // These methods enumerate the available items.  Returns NULL if no (or no more) items are
    // available or an error occurs.  The former case can be distinguished from the latter by
    // checking GetLastError() for ERROR_NO_MORE_ITEMS.
    //
    // The strings returned by these methods remain valid only until the next call to either
    // method.  Note that Container_GetFirstUrl() can be called at any time to reinitiate the
    // enumeration process.

    virtual UrlApi* TIVO_API_METHOD Container_GetFirstUrl() = 0;
    virtual UrlApi* TIVO_API_METHOD Container_GetNextUrl() = 0;


    // This method can be called from any thread to interrupt processing occurring within the
    // Container_GetFirstUrl() or Container_GetNextUrl() method.  The call in progress at the
    // time will return NULL and GetLastError() will return ERROR_REQUEST_ABORTED.
    //
    // This method has no effect if no Container_GetFirstUrl() or Container_GetNextUrl() call
    // is in progress.

    virtual void    TIVO_API_METHOD Container_Abort() = 0;


    // These methods convert between a unique (within the container) "token" for an item and
    // the corresponding URL.  This string should not contain any slashes or other characters
    // illegal for URL path component name.  Returns NULL if an error occurs.
    //
    // Tokens will be used when building a URL that refers to a particular item.  For example,
    // "http://SOME_PC/music/cool/song.mp3".  Here, "cool" is the token for some container and
    // "song.mp3" is the token for the MPEG audio file "song.mp3".
    //
    // The strings returned by these methods remain valid only until the next call to either
    // method.

    virtual LPCSTR  TIVO_API_METHOD Container_TokenFromUrl(UrlApi* url) = 0;
    virtual UrlApi* TIVO_API_METHOD Container_UrlFromToken(LPCSTR token) = 0;


    // These methods allow clients to "sign up" for notifications about changes occuring within
    // the container (and all containers contained by the container when the 'recurse' is TRUE).
    // Returns FALSE if an error occurs.
    //
    // Note that multiple requests to register the same callback are ignored, but return TRUE.

    virtual BOOL    TIVO_API_METHOD Container_Register
                    (
                        ContainerCallbackApi*   callback,
                        BOOL                    recurse

                    ) = 0;

    virtual void    TIVO_API_METHOD Container_Unregister(ContainerCallbackApi* callback) = 0;

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ContainerCallbackApi
//
//  This interface is optionally implemented by the client of ContainerApi.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct ContainerCallbackApi: TivoApi::Api
{

    // Occurs whenever the container associated with the callback is modified.

    virtual void    TIVO_API_METHOD Container_OnNotify
                    (
                        ContainerApi*   container,
                        DWORD           reason,
                        UrlApi*         url

                    ) = 0;

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Possible values for the 'reason' parameter of Container_OnNotify().
//
//////////////////////////////////////////////////////////////////////////////////////////////////

enum ContainerNotifyCode
{
    ContainerItemAdded, // indicated by 'url'.
    ContainerItemRemoved, // indicated by 'url'.
    ContainerItemChanged // indicated by 'url'.

/* TBD: Add something like this for scouring operations?

    ContainerProgressBegin,
    ContainerProgressStep,
    ContainerProgressEnd,

*/

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// This interface is retrieved via Host_GetApi() and provides a way to "filter" the contents for
// any container URL.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct FilterContainerApi: ContainerApi
{

    // This method must be called before any others.
    //
    // The supplied comma-delimted "mask" will cause items only of the indicates type(s) to
    // be enumerated by Container_GetFirstUrl() and Container_GetNextUrl().  The behavior and
    // format of this parameter is the same as that defined for the Filter parameter of the
    // QueryContainer command in the "Calypso Server Protocol Specification".
    //
    // When ContainerRecurse is passed to Filter_Initialize(), containers that do not contain
    // any items(s) matching the filter (anywhere within their sub-hierarchy) are excluded.
    //
    // Returns FALSE on error.

    virtual BOOL    TIVO_API_METHOD Filter_Initialize(UrlApi* url, LPCSTR mask, DWORD flags) = 0;


    // If called, the supplied prefix will be added to every URL returned by the container.
    // This is useful for routing Media_GetApi() requests for all such URLs to a specific
    // module.  Returns FALSE on error.

    virtual BOOL    TIVO_API_METHOD Filter_SetUrlPrefix(LPCSTR prefix) = 0;

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Possible OR-able values for the 'flags' parameter of Filter_SetSourceFormatMask() and
// Filter_SetTypeMask().
//
//////////////////////////////////////////////////////////////////////////////////////////////////

enum FilterContainerFlags
{
    ContainerRecurse        = 0x0001,
    ContainerFollowLinks    = 0x0002,
};


#endif // INCLUDE_GUARD