/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_9A00169B_3BE4_4FFF_8182_B34BDF07A336
#define INCLUDE_GUARD_9A00169B_3BE4_4FFF_8182_B34BDF07A336

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// DetailsApi.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "Api/UrlApi.h"


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// This interface is retrieved via Media_GetApi().  The list of details available depends on the
// specified URL.
//
// See the "Calpyso Server Protocol Specification" for a list of details.  The keys returned by
// DetailsApi should match the XML meta-data element names defined in the specification.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct DetailsApi: TivoApi::Api
{

    // These methods enumerate the available details.  Returns NULL if no (or no more) details
    // are supported or if an error occurs.  The former case can be distinguished from the
    // latter by checking GetLastError() for ERROR_NO_MORE_ITEMS.
    //
    // No key should occur more than once within a particular set of details.

    virtual LPCSTR  TIVO_API_METHOD Details_GetFirstKey() = 0;
    virtual LPCSTR  TIVO_API_METHOD Details_GetNextKey() = 0;


    // Returns text associated with the given detail.  Returns NULL if the requested detail
    // is not available or if an error occurs.  The former case can be distinguished from the
    // latter by checking GetLastError() for ERROR_NOT_FOUND.

    virtual LPCSTR  TIVO_API_METHOD Details_QueryValue(LPCSTR key) = 0;

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// This interface is retrieved via Host_GetApi() and provides a way to "filter" the details for
// any URL.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct FilterDetailsApi: DetailsApi
{

    // This method must be called before any others.  Returns FALSE on error.

    virtual BOOL        TIVO_API_METHOD Filter_Initialize(UrlApi* url) = 0;


    // Prevents the named key, if valid for the URL passed to Filter_Initialize(), from being
    // returned by Details_GetFirstKey() and Details_GetNextKey().  Returns FALSE on error.

    virtual BOOL        TIVO_API_METHOD Filter_HideKey(LPCSTR key) = 0;


    // Causes Details_QueryValue() to return the supplied value, "overriding" that normally
    // returned for the URL passed to Filter_Initialize().  If the 'value' parameter is NULL,
    // the associated key is "deleted".  Returns FALSE on error.

    virtual BOOL        TIVO_API_METHOD Filter_SetValue(LPCSTR key, LPCSTR value) = 0;

};


#endif // INCLUDE_GUARD
