/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_BCC1ADA5_5C8F_492A_8644_A8C59406021D
#define INCLUDE_GUARD_BCC1ADA5_5C8F_492A_8644_A8C59406021D

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// FormatApi.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "TivoApi/Api.h"

#include "Windows.h"


//////////////////////////////////////////////////////////////////////////////////////////////////

struct FormatType;
struct DataApi;
struct UrlApi;


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// FormatApi
//
//  This interface is retrieved via Host_GetApi(), and can also optionally be supported by media
//  Modules via Tivo_GetApi() or Group_GetApi().  In this case, the host program will return the
//  combined Module-provided information via its implementation of FormatApi in addition to that
//  provided by Windows.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct FormatApi: TivoApi::Api
{

    // Returns the total number of formats.  This method should not return zero.

    virtual DWORD       TIVO_API_METHOD Format_GetNumTypes() = 0;


    // Returns information about the format indicated by the supplied zero-based index.  If
    // an error occurs, returns FALSE.

    virtual BOOL        TIVO_API_METHOD Format_GetType(DWORD index, FormatType* type) = 0;


    // Returns the zero-based index of the format of the data referred to by the specified URL.
    // The data supplied will always contain the very first byte of that referred to by the URL,
    // but not always the entire content (perhaps the first 1000 bytes or so).  Returns -1 when
    // the format cannot be determined or if an occurs.  The former case can be distinguished
    // from the latter by checking GetLastError() for ERROR_UNSUPPORTED_TYPE.
    //
    // Note that either one the parameters might be NULL, but never both.

    virtual LONG        TIVO_API_METHOD Format_SniffData(UrlApi* url, DataApi* data) = 0;

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// FormatType
//
//  Each structure should contain the complete MIME type, a comma-delimited list of the typical
//  Windows file extention(s), and a brief human-readable description for the associated format.
//  For example, "image/jpeg", ".jpg,.jpeg", and "JPEG Image File".
//
//  The MimeType member should never be NULL.  However, the other members can be NULL if this
//  information is not available.  This first extension listed in FileExtensions should be the
//  "primary".
//
//  All the returned strings returned should be "static", remaining valid until the originating
//  FormatApi instance is destroyed.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct FormatType
{
    LPCSTR MimeType;
    LPCSTR FileExtensions;
    LPCSTR DisplayName;
};


#endif // INCLUDE_GUARD