/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_25A9ED7D_FA6F_4B8F_80BC_3FE7687C57E7
#define INCLUDE_GUARD_25A9ED7D_FA6F_4B8F_80BC_3FE7687C57E7

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ImageApi.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "TivoApi/Api.h"

#include "Windows.h"


//////////////////////////////////////////////////////////////////////////////////////////////////

struct DataApi;


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ImageApi
//
//  Every DLL housing one or more image conversion modules should support this interface via
//  Media_GetApi().
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct ImageApi: TivoApi::Api
{

    // Requests that the data generated by Image_GetDatApi() be scaled to fit in the given
    // number of pixels.  Returns FALSE is an error occurs.

    virtual BOOL        TIVO_API_METHOD Image_Scale(DWORD width, DWORD height) = 0;


    // Requests that the data generated by Image_GetDatApi() be rotated the given number of
    // degrees.  If the number is positive, the rotation should be performed in a clockwise
    // manner.  If negative, counter-clockwise.  Returns FALSE if an error occurs.
    //
    // For example, Image_Rotate(180) should turn the image upsidedown and Image_Rotate(-90)
    // should "tip" the picture over onto its left side.

    virtual BOOL        TIVO_API_METHOD Image_Rotate(LONG amount) = 0;


    // Requests that the data generated by Image_GetDatApi() have the given quality level
    // (relative to the quality of the source data), indicating using the values defined by
    // the QualityLevel enum.  Returns FALSE if an error occurs.

    virtual BOOL        TIVO_API_METHOD Image_SetQuality(DWORD level) = 0;


    // Requests that the data generated by Image_GetDatApi() take into account the supplied
    // "pixel shape".  Any ratio can be passed.  For example, Image_SetPixelShape(2, 1) can
    // be used to specify pixels that are twice as wide as they are tall, Because any ratio
    // can be used, Image_SetPixelShape(666, 333) would be equivalent.  Returns FALSE if an
    // error occurs.

    virtual BOOL        TIVO_API_METHOD Image_SetPixelShape(DWORD width, DWORD height) = 0;


    // Renders binary data for the image in the given format, specified as a MIME type, which
    // should always match one of the entries in the list returned by the associated module's
    // Media_GetTargetFormats() method.  Returns NULL if the requested format is not supported
    // or an error occurs.  The former case can be distinguished from the latter by checking
    // GetLastError() for ERROR_UNSUPPORTED_TYPE.
    //
    // For example, "image/jpeg" will be specified when JPEG data is desired.

    virtual DataApi*    TIVO_API_METHOD Image_GetDataApi(LPCSTR format) = 0;

};


#endif // INCLUDE_GUARD