/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_E3972D99_BD73_4D2D_91F8_4F06BD81B79D
#define INCLUDE_GUARD_E3972D99_BD73_4D2D_91F8_4F06BD81B79D

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// MediaApi.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "Api/ModuleApi.h"
#include "Api/UrlApi.h"


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// MediaApi
//
//  Every DLL housing one or more media modules should support this interface via Tivo_GetApi()
//  or Group_GetApi().  The host program also supports this interface via Host_GetApi(), which
//  provides information about all media support.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct MediaApi: TivoApi::Api
{

    // Returns comma-delimited lists of MIME types, indicating the set of "input" data formats
    // supported by the module.  This method cannot return NULL and should always return a list
    // containing at least one entry.

    virtual LPCSTR          TIVO_API_METHOD Media_GetSourceFormats() = 0;


    // Returns comma-delimited lists of MIME types, indicating the set of "output" data formats
    // supported by the module.  If the module does not support data generation (for example, a
    // module that only enumerates conatiner contents via ContainerApi), this method can return
    // NULL.

    virtual LPCSTR          TIVO_API_METHOD Media_GetTargetFormats() = 0;


    // This function can be used to retrieve the following interfaces, which behave according
    // to the specified URL.
    //
    //  DetailsApi
    //  LinksApi
    //  ImageApi
    //  AudioApi
    //  ContainerApi
    //
    // Returns NULL if the requested interface is not supported, not supported for the given
    // URL, or if an error occurs.  The two former cases can be distinguished from the latter
    // by checking GetLastError() for ERROR_INVALID_PARAMETER and ERROR_DATATYPE_MISMATCH,
    // respectively.

    virtual TivoApi::Api*   TIVO_API_METHOD Media_GetApi(LPCSTR type_id, UrlApi* url) = 0;

};


#endif // INCLUDE_GUARD
