/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_66A76A6D_C89C_4D14_84F2_257D84C07865
#define INCLUDE_GUARD_66A76A6D_C89C_4D14_84F2_257D84C07865

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// MediaClassApi.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "Api/UrlApi.h"


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// MediaClassApi
//
//  Every DLL housing one or more media classification Modules should support this interface
//  via Tivo_GetApi() or Group_GetApi().
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct MediaClassApi: TivoApi::Api
{

    // This method identifies the container of available content for the class.  If the class
    // does not support user configuration, this method returns NULL.
    //
    // The title of this container should be a short human-readable description of the content.
    // For example, "My Computer".  This string need not be unique.

    virtual UrlApi*         TIVO_API_METHOD Class_GetAvailableUrl() = 0;


    // This method identifies the container of published content for the class.
    //
    // The title of this container should be a short human-readable description of the class,
    // which need not be unique.  For example, "Music on Jeff's PC".  This method should not
    // return NULL.

    virtual UrlApi*         TIVO_API_METHOD Class_GetPublishedUrl() = 0;


    // These methods test whether an item can be moved into or out of the "published" root
    // container.  Note that URL given to Class_Publish() should refer to the item to be
    // published, but the URL passed to Class_Unpublish() should refer to the entry created
    // for the published item within the "published" root container.  Returns FALSE if the
    // item cannot be published/unpublished or if an error occurs.  The former case can be
    // distinguished from the latter by checking GetLastError() for ERROR_DATATYPE_MISMATCH.

    virtual BOOL            TIVO_API_METHOD Class_CanPublish(UrlApi* url) = 0;
    virtual BOOL            TIVO_API_METHOD Class_CanUnpublish(UrlApi* url) = 0;


    // These methods move "available" content into and out of the "published" root container,
    // but should not be called if Class_GetAvailableUrl() returns NULL.  Note that URL given
    // to Class_Publish() should refer to the item to be published, but the URL passed to
    // Class_Unpublish() should refer to the entry created for the published item within the
    // "published" root container (returned by Class_Publish()).  Returns NULL if an error
    // occurs.

    virtual UrlApi*         TIVO_API_METHOD Class_Publish(UrlApi* url, LPCSTR title = NULL) = 0;
    virtual UrlApi*         TIVO_API_METHOD Class_Unpublish(UrlApi* url) = 0;

};


#endif // INCLUDE_GUARD