/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_3D3F629E_9BD1_40A2_B12C_370A518FA049
#define INCLUDE_GUARD_3D3F629E_9BD1_40A2_B12C_370A518FA049

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ModuleApi.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "TivoApi/Api.h"

#include "Windows.h"


//////////////////////////////////////////////////////////////////////////////////////////////////

struct HostProgramApi;


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ModuleApi
//
//  This interface is typically retrieved from the Tivo_GetApi() entry-point of the housing DLL.
//  However, the host program will always first request ModuleGroupApi, which allows the DLL to
//  indicate support for multiple Modules.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct ModuleApi: TivoApi::Api
{

    // This method is always called before all others, and before any other API is requested.
    // Returns FALSE if the Module cannot be initialized, after which no other methods will be
    // called except Api_AddRef() and/or Api_Release().

    virtual BOOL            TIVO_API_METHOD Module_Initialize(HostProgramApi* host) = 0;


    // Returns a human-readable description of the Module, such as "WinAmp Playlist Support".
    // This function should not return NULL.

    virtual LPCSTR          TIVO_API_METHOD Module_GetDisplayName() = 0;


    // Returns an unchanging internal name for the Module, such as "TivoPhotos".  This string
    // should not contain any slashes or other characters illegal for a URL, Windows file name,
    // registry key.  A plain alpha-numeric string is probably the best idea.  This function
    // should not return NULL.
    //
    // For media classification Modules, this string will be used when building URLs referring
    // to class content.  For example, "http://SOME_PC/TivoPhotos/Cool/Picture.jpg".

    virtual LPCSTR          TIVO_API_METHOD Module_GetInternalName() = 0;

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ModuleGroupApi
//
//  This interface is retrieved from the Tivo_GetApi() entry-point of the housing DLL.  If this
//  interface is supported by the DLL, the host program will use Group_GetApi() to request APIs
//  separately for each module indicated by Group_GetNumModules().  Otherwise, the host assumes
//  the DLL houses only a single module and will use Tivo_GetApi() directly instead.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct ModuleGroupApi: TivoApi::Api
{

/* Do we need this ???

    // This method is always called before all others.  Returns FALSE if the DLL cannot be
    // initialized, after which no other methods will be called other than Api_AddRef() and/or
    // Api_Release().

    virtual BOOL            TIVO_API_METHOD Group_Initialize(HostProgramApi* host) = 0;
*/


    // Returns the number of modules housed in the DLL.

    virtual DWORD           TIVO_API_METHOD Group_GetNumModules() = 0;


    // Returns NULL if the requested interface is not supported by the indexed module, or
    // if an error occurs.  The former case can be distinguised from the latter by checking
    // GetLastError() for ERROR_INVALID_PARAMETER.

    virtual TivoApi::Api*   TIVO_API_METHOD Group_GetApi(LPCSTR type_id, DWORD index) = 0;

};


#endif // INCLUDE_GUARD
