/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_1E3550A3_E1E3_4D1D_9B97_1BD53F65B2C3
#define INCLUDE_GUARD_1E3550A3_E1E3_4D1D_9B97_1BD53F65B2C3

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// UrlApi.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "TivoApi/Api.h"

#include "Windows.h"


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// UrlApi
//
//  This interface is retrieved via Host_GetApi() and Url_GetUrlApi().
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct UrlApi: TivoApi::Api
{

    // This method must be called before all others.  Parses the supplied URL (specified using
    // escaped or unescaped text), making the individual elements of the syntax available via
    // the other members.  If an error occurs (such as invalid URL syntax), returns FALSE.
    //
    // All other methods can be assumed to use only unescaped text (except for Url_GetText()).
    // The strings returned remain valid until the next call to any method.

    virtual BOOL        TIVO_API_METHOD Url_Initialize(LPCSTR url = NULL) = 0;


    // Gets the "scheme" specified in the URL.  For example, "http".  Returns NULL if an error
    // occurs or the syntax element is not present.  The latter case can be distinguished from
    // the former by checking GetLastError() for ERROR_NOT_FOUND.

    virtual LPCSTR      TIVO_API_METHOD Url_GetScheme() = 0;


    // Gets the "host" machine name.  For example, "www.tivo.com".  Returns NULL if an error
    // occurs or the syntax element is not present.  The latter case can be distinguished from
    // the former by checking GetLastError() for ERROR_NOT_FOUND.

    virtual LPCSTR      TIVO_API_METHOD Url_GetHostName() = 0;


    // Gets the document path.  For example, "Foo\Bar\Baz\index.html".  Returns NULL if an
    // error occurs or the syntax element is not present.  The latter case can be distinguished
    // from the former by checking GetLastError() for ERROR_NOT_FOUND.

    virtual LPCSTR      TIVO_API_METHOD Url_GetDocument() = 0;


    // Returns the number of parameters in the URL.  Returns zero (0) if the URL does not have
    // any parameters.

    virtual DWORD       TIVO_API_METHOD Url_GetNumParams() = 0;


    // Enumerates any parameters present in the URL.  Returns NULL if an error occurs (or if a
    // parameter's value is empty).  The latter case can be distinguished from the former by
    // checking GetLastError() for ERROR_NOT_FOUND.

    virtual LPCSTR      TIVO_API_METHOD Url_GetParamName(DWORD index) = 0;
    virtual LPCSTR      TIVO_API_METHOD Url_GetParamValue(DWORD index) = 0;


    // Gets the value of a specific parameter.  Returns NULL if the parameter isn't present or
    // if an error occurs.  The former case can be distinguished from the latter by checking
    // GetLastError() for ERROR_NOT_FOUND.

    virtual LPCSTR      TIVO_API_METHOD Url_QueryParamValue(LPCSTR name) = 0;


    // Get the complete URL in text form.  Returns NULL if the URL is empty or if an error
    // occurs.  The former case can be distinguished from the latter by checking GetLastError()
    // for ERROR_NOT_FOUND.

    virtual LPCSTR      TIVO_API_METHOD Url_GetText() = 0;


    // Get the relative URL (minus the scheme and host name elements) in text form.  Returns
    // NULL if the URL is empty or if an error occurs.  The former case can be distinguished
    // from the latter by checking GetLastError() for ERROR_NOT_FOUND.

    virtual LPCSTR      TIVO_API_METHOD Url_GetRelativeText() = 0;

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// UrlBuilderApi
//
//  This interface is retrieved via Host_GetApi().
//
//////////////////////////////////////////////////////////////////////////////////////////////////

struct UrlBuilderApi: TivoApi::Api
{

    // This method must be called before all others.  An existing UrlApi instance can be passed
    // to provide the initial settings of the URL to be built.  Returns NULL if an error occurs.
    //
    // Note that each instance of UrlBuilderApi can be "re-initialized" as many times as desired
    // (to clear the most recent, or start with other initial settings) by executing this member
    // again.

    virtual BOOL        TIVO_API_METHOD Url_Initialize(UrlApi* url = NULL) = 0;


    // Sets the "scheme" for the URL to be built.  Returns FALSE if an error occurs.

    virtual BOOL        TIVO_API_METHOD Url_SetScheme(LPCSTR scheme) = 0;


    // Sets the "host" machine name.  Returns FALSE if an error occurs.

    virtual BOOL        TIVO_API_METHOD Url_SetHostName(LPCSTR name) = 0;


    // Sets the document path.  Returns FALSE if an error occurs.

    virtual BOOL        TIVO_API_METHOD Url_SetDocument(LPCSTR path) = 0;


    // Sets the value of a specific parameter.  Setting a parameter to NULL clears it from the
    // URL.  Returns FALSE if an error occurs.
    //
    // Note that calling Url_SetParamValue(NULL, NULL) will clear ALL parameters.

    virtual BOOL        TIVO_API_METHOD Url_SetParamValue(LPCSTR name, LPCSTR value) = 0;


    // Builds an URL using the most recent settings.  Returns a new UrlApi instance on success
    // or NULL if an error occurs.

    virtual UrlApi*     TIVO_API_METHOD Url_GetUrlApi() = 0;

};


#endif // INCLUDE_GUARD
