/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ModuleErrorUtils.h
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef INCLUDE_GUARD_00774C67_FC63_4757_ABC4_A890CD260699
#define INCLUDE_GUARD_00774C67_FC63_4757_ABC4_A890CD260699

#include <windows.h>
#include <new>

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// WinErrorException
//
//////////////////////////////////////////////////////////////////////////////////////////////////
struct WinErrorException
{
    WinErrorException(DWORD error) :
        Error(error)
    {
    }

    const DWORD Error;
};
//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Call this function when you expect to receive an error code after a failed function call but
// want to make sure you don't end up with ERROR_SUCCESS (in case the function doesn't
// call SetLastError() properly).
//
//////////////////////////////////////////////////////////////////////////////////////////////////
inline DWORD GetLastErrorMustFail()
{
    DWORD error = GetLastError();

    if (error == ERROR_SUCCESS)
        return ERROR_GEN_FAILURE;

    return error;
}
//////////////////////////////////////////////////////////////////////////////////////////////////
//
// These functions throw exceptions of type WinErrorException.
//
//////////////////////////////////////////////////////////////////////////////////////////////////
inline void ThrowWinError(DWORD error)
{
    throw WinErrorException(error);
}
//////////////////////////////////////////////////////////////////////////////////////////////////
inline void ThrowLastWinError()
{
    ThrowWinError(GetLastErrorMustFail());
}
//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Note that the dummy initialization of 'catch_error' squelches "used before definition"
// warnings caused by TIVO_API_CATCH's dummy reference.
//
//////////////////////////////////////////////////////////////////////////////////////////////////
#define TIVO_API_TRY \
    { \
        DWORD catch_error = ERROR_SUCCESS; \
        \
        catch_error; \
        try \
        {
//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Note that the dummy 'catch_error' reference squelches "delcared but not used" warnings.
//
//////////////////////////////////////////////////////////////////////////////////////////////////
#define TIVO_API_CATCH \
        } \
        \
        catch ( ... ) { } \
        \
        catch_error; \
    }
//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Note that we must use the temporary variable 'catch_error' to transmit the error value to
// the SetLastError() call just before returning because otherwise the destruction of the
// damn exception objects themselves (as we leave the scope of the catch block) will clobber
// the error value (if we were to call SetLastError() from within each catch block instead).
//
//////////////////////////////////////////////////////////////////////////////////////////////////
#define TIVO_API_CATCH_RETURN(result) \
        } \
        \
        catch (const std::bad_alloc&) { catch_error = ERROR_NOT_ENOUGH_MEMORY; } \
        \
        catch (const WinErrorException& x) { catch_error = x.Error; } \
        \
        catch ( ... ) { catch_error = ERROR_INTERNAL_ERROR; } \
        \
        SetLastError(catch_error); \
        \
        return result; \
    }
//////////////////////////////////////////////////////////////////////////////////////////////////

#endif // INCLUDE_GUARD