/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// AudioDetails.h
//
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef INCLUDE_GUARD_81957710_4033_4919_9C96_E70289EB79A2
#define INCLUDE_GUARD_81957710_4033_4919_9C96_E70289EB79A2

#include <windows.h>
#include <string>
#include <memory>

#include "Api/DataApi.h"
#include "Api/DetailsApi.h"
#include "Api/DetailsHelpApi.h"
#include "Api/FileViewApi.h"
#include "Api/CacheApi.h"
#include "Api/UrlApi.h"

#include "TivoApi/ApiRef.h"
#include "TivoApi/ApiHelp.h"
#include "TivoApi/Dll/DllHelp.h"
#include "Common/ApiRefs.h"
#include "Common/ModuleErrorUtils.h"

#include "AudioModule.h"
#include "MpegAudio.h"
#include "id3v2.h"


////////////////////////////////////////////////////////////////////////////////

using namespace TivoApi;

typedef enum
{
    // Control
    kdtUnknown       = 0,

    // Generic
    kdtType,
    kdtTitle,
    kdtSourceFormat,
    kdtSourceLocation,
    kdtFileName,
    kdtCreationDate,
    kdtLastChangeDate,
    kdtLastAccessDate,
    kdtSourceSize,
    kdtKeywords,
    kdtCopyright,

    // Audio specific
    kdtSourceBitRate,
    kdtSourceSampleRate,
    kdtMusicGenre,
    kdtArtistName,
    kdtAlbumTitle,
    kdtDuration,
    kdtAccurateDuration,
    kdtAlbumYear,
    kdtRecordLabel,
    kdtSongTitle,

} KeyDataType;

////////////////////////////////////////////////////////////////////////////////
class StringDataApi : public RefCounted < Instanced <StringDataApi, Implements<DataApi> > >
{
    std::string      m_string;

    public:

    StringDataApi(LPCSTR str) : m_string(str)
    {
    }

    ~StringDataApi()
    {
    }


    LPCVOID     TIVO_API_METHOD Data_Get()
    {
        return m_string.c_str();
    }

    DWORD       TIVO_API_METHOD Data_GetSize()
    {
        return m_string.length();
    }
};
////////////////////////////////////////////////////////////////////////////////
class AudioDetails : public RefCounted < Instanced <AudioDetails, Implements<DetailsApi> > >
{
    struct KeyData
    {
        KeyDataType        type;
        LPCSTR             key;
        bool               supported;
    };

    UrlRef                 m_url_api;
    DetailsHelpRef         m_dhelp_api;
    CacheRef               m_cache_api;
    static KeyData         s_key_array[];
    int                    m_key_idx;
    std::auto_ptr<Mp3Info> m_mp3;
    std::auto_ptr<id3v2>   m_id3;
    std::string            m_location;
    std::string            m_value;
    bool                   m_accuracy;
    bool                   m_error;
    std::string            m_duration;
    DWORD                  m_access_timeout;

    static const int       c_timeout_value = (1000 * 30); // 30 seconds

    KeyDataType GetTypeFromString(LPCSTR str);
    std::string GetFileNameFromUrl();

    Mp3Info *InitMp3(Mp3Info *&mp3, bool full = false);
    id3v2 *InitId3(id3v2 *&id3, bool full = false);
    void ScanFileImage(bool full, bool mp3_or_id3);
    std::string CalculateDuration(bool accurate);
    std::string AudioDetails::FindDuration(KeyDataType type);

    public:

    // Ctor & Dtor
    AudioDetails(UrlApi* raw_url);
    ~AudioDetails();

    // DataDetailsApi methods
    LPCSTR  TIVO_API_METHOD Details_GetFirstKey();
    LPCSTR  TIVO_API_METHOD Details_GetNextKey();
    LPCSTR  TIVO_API_METHOD Details_QueryValue(LPCSTR key);

};
////////////////////////////////////////////////////////////////////////////////
#endif // INCLUDE_GUARD
