/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#include <dir.h>
#include <stdio.h>
#include <string.h>
#include <windows.h>
#include "AudioModule.h"
#include "AudioDetails.h"
#include "Audio.h"

////////////////////////////////////////////////////////////////////////////////
extern "C" { extern HINSTANCE _hInstance; };  // "C"
////////////////////////////////////////////////////////////////////////////////
Register < AudioModule, ModuleApi> AudioModuleReg;
Register < AudioModule, MediaApi>  AudioMediaReg;
Register < AudioModule, FormatApi> AudioFormatReg;
////////////////////////////////////////////////////////////////////////////////
HostProgramApi*  BaseRoot::m_host_program_api;
BaseRoot*        BaseRoot::m_self;
////////////////////////////////////////////////////////////////////////////////
AudioModule::AudioModule()
{
}
////////////////////////////////////////////////////////////////////////////////
AudioModule::~AudioModule()
{
}
////////////////////////////////////////////////////////////////////////////////
BOOL    TIVO_API_METHOD AudioModule::Module_Initialize(HostProgramApi* host)
{
    TIVO_API_TRY

    InitBaseModule(host);

    return true;

    TIVO_API_CATCH_RETURN(false)
}
////////////////////////////////////////////////////////////////////////////////
LPCSTR  TIVO_API_METHOD AudioModule::Module_GetInternalName()
{
    TIVO_API_TRY

    return "TivoMp3";

    TIVO_API_CATCH_RETURN(NULL)
}
////////////////////////////////////////////////////////////////////////////////
LPCSTR  TIVO_API_METHOD AudioModule::Module_GetDisplayName()
{
    TIVO_API_TRY

    return "TiVo MP3 Audio Support";

    TIVO_API_CATCH_RETURN(NULL)
}
////////////////////////////////////////////////////////////////////////////////
LPCSTR  TIVO_API_METHOD AudioModule::Media_GetSourceFormats()
{
    TIVO_API_TRY

    return "audio/mpeg,audio/x-mpeg,audio/x-mp3,audio/mp3";

    TIVO_API_CATCH_RETURN(NULL)
}
////////////////////////////////////////////////////////////////////////////////
LPCSTR  TIVO_API_METHOD AudioModule::Media_GetTargetFormats()
{
    TIVO_API_TRY

    return "audio/mpeg";

    TIVO_API_CATCH_RETURN(NULL)
}
////////////////////////////////////////////////////////////////////////////////
Api*    TIVO_API_METHOD AudioModule::Media_GetApi(LPCSTR type_id, UrlApi* raw_url)
{
    TIVO_API_TRY

    UrlRef url_api = raw_url;

    // This function can get called by many different threads asking for Apis
    // for various purposes e.g. several tcd's asking to play audio

    // FUTURE:
    // This would be a good pick-off point to provide some mru cache of
    // "stuff", maybe the MpegAudio objects for each url

    if (strcmp(type_id, "AudioApi") == 0)
        return new Audio(url_api);
    else
    {
        if (strcmp(type_id, "DetailsApi") == 0)
        {
            DWORD fattr;

            fattr = GetFileAttributes(SafeString(url_api->Url_GetDocument()).c_str());

            if (fattr == 0xFFFFFFFF)
            {
                // This file cannot be located, redirect to the broken
                // link details
                return this->GetBrokenDetailsApi(url_api);
            }

            return new AudioDetails(url_api);
        }
    }

    SetLastError(ERROR_INVALID_PARAMETER);
    return NULL;

    TIVO_API_CATCH_RETURN(NULL)
}
////////////////////////////////////////////////////////////////////////////////
DWORD   TIVO_API_METHOD AudioModule::Format_GetNumTypes()
{
    TIVO_API_TRY

    return 1;

    TIVO_API_CATCH_RETURN(0)
}
////////////////////////////////////////////////////////////////////////////////
BOOL    TIVO_API_METHOD AudioModule::Format_GetType(DWORD index, FormatType* type)
{
    TIVO_API_TRY

    // We only support one type: index 0
    if (index != 0)
        return false;

    type->MimeType = "audio/mpeg";
    type->FileExtensions = ".mp3";
    type->DisplayName = "MPEG Layer 3 Audio";

    return true;

    TIVO_API_CATCH_RETURN(false)
}
////////////////////////////////////////////////////////////////////////////////
LONG    TIVO_API_METHOD AudioModule::Format_SniffData(UrlApi* raw_url, DataApi* raw_data)
{
    TIVO_API_TRY

    DWORD fattr;
    DataRef the_data = raw_data;
    UrlRef  url_api = raw_url;

    // JJP: Hack for now to allow internet radio
    if (strcmp (SafeString(url_api->Url_GetScheme()).c_str(), "http") == 0)
    {
        // For now, we'll assume that all http urls point to audio/mpeg.

        return 0;
    }
    else
    if (strcmp (SafeString(url_api->Url_GetScheme()).c_str(), "file") != 0)
    {
        // We only support the file:/// scheme

        SetLastError(ERROR_UNSUPPORTED_TYPE);
        return -1;
    }

    // We only care about files with the mp3 extension
    if (stricmp(ExtractFileExt(SafeString(url_api->Url_GetDocument())).c_str(),".mp3") != 0)
    {
        SetLastError(ERROR_UNSUPPORTED_TYPE);
        return -1;
    }

    // We'll assume that this is an mp3 file without spending the time needed
    // to analyze the whole file.  So we'll return the index of this mime type
    // (in this case there is only one so we return 0).
    return 0;


    TIVO_API_CATCH_RETURN(-1)
}
////////////////////////////////////////////////////////////////////////////////
