/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

////////////////////////////////////////////////////////////////////////////////
//
// AudioStream.h
//
// Definition for the AudioStream and AudioData classes
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef INCLUDE_GUARD_47C91AA8_2F39_4B28_B209_16557E0B2715
#define INCLUDE_GUARD_47C91AA8_2F39_4B28_B209_16557E0B2715

#include <windows.h>

#include "Api/DataApi.h"
#include "Api/StreamApi.h"
#include "Api/FileViewApi.h"

#include "TivoApi/ApiRef.h"
#include "TivoApi/ApiHelp.h"
#include "TivoApi/Dll/DllHelp.h"
#include "Common/Apirefs.h"
#include "Common/ModuleErrorUtils.h"

#include "AudioModule.h"

using namespace TivoApi;

////////////////////////////////////////////////////////////////////////////////
class AudioData : public RefCounted < Instanced <AudioData, Implements<DataApi> > >
{
    FileViewRef             m_file_view;
    DWORD                   m_begin;
    DWORD                   m_length;

    public:
    //---------------------------------------------------------
    AudioData(FileViewApi *fva, DWORD begin, DWORD length) :
        m_file_view(fva),
        m_begin(begin),
        m_length(length)
    {
    }
    //---------------------------------------------------------
    ~AudioData()
    {
    }
    //---------------------------------------------------------
    LPCVOID     TIVO_API_METHOD Data_Get()
    {
        const char *p = (const char *)m_file_view->Data_Get();

        return (LPCVOID) &p[m_begin];
    }
    //---------------------------------------------------------
    DWORD       TIVO_API_METHOD Data_GetSize()
    {
        return m_length;
    }
    //---------------------------------------------------------
};
////////////////////////////////////////////////////////////////////////////////
class AudioStream : public RefCounted < Instanced <AudioStream, Implements<StreamApi> > >
{
    UrlRef                  m_url_api;
    FileViewRef             m_file_view;
    DWORD                   m_begin;
    DWORD                   m_length;
    bool                    m_eos;

    public:

    // Ctor & Dtor
    AudioStream(UrlApi* raw_url, FileViewApi *fva, DWORD begin, DWORD length);
    ~AudioStream();

    // DataStreamApi methods
    LONG              TIVO_API_METHOD Stream_GetTotalSize();
    DataApi*          TIVO_API_METHOD Stream_GetNextDataApi();
    void              TIVO_API_METHOD Stream_Reset();
};

////////////////////////////////////////////////////////////////////////////////

#endif // INCLUDE_GUARD
