/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_C1AFCA17_C09E_4076_BC19_B67D75C9EE69
#define INCLUDE_GUARD_C1AFCA17_C09E_4076_BC19_B67D75C9EE69

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ApiManager.h
//
//  Class for managing the loading and unloading of Api modules and Api instances.  Currently, a
//  given DLL should only be loaded by one ApiManager instance at a time.
//
//
// Copyright (c) 2001 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "TivoApi/ApiRef.h"
#include "TivoApi/ApiHelp.h"
#include "TivoApi/DllApi.h"

#include <typeinfo>
#include <vector>
#include <map>


namespace TivoApi {


//////////////////////////////////////////////////////////////////////////////////////////////////

class ApiManager:

    private TivoApi::RefNeutral<DllCallbackApi>

{
    public:

        // Creators

        ApiManager();

        ~ApiManager();


        // Accessors

        const char*                     GetApiModulePath(ApiRef<Api> api) const;


        // Manipulators

        ApiRef<Api>                     GetApi(const char* type_id, const char* path);

        typedef std::vector< ApiRef<Api> > ApiArray;

        ApiArray                        SearchApi(const char* type_id, const char* path);

        void                            UnloadDlls();


        // Internals

        class Module;


    private:

        // DllCallbackApi implementation.

        virtual void TIVO_API_METHOD    Dll_OnApiRelease(Api* old_api);


        // Internals

        typedef std::vector<Module*> ModuleArray;
        typedef std::map<Api*, Module*, std::less<Api*> > ApiToModuleMap;

        Module*                         GetModuleByPath(const char* path);
        Module*                         GetModuleByApi(Api* api) const;

        ModuleArray                     m_modules;
        ApiToModuleMap                  m_api_map;
        mutable char                    m_temp[MAX_PATH];

};


} // TivoApi


#endif // INCLUDE_GUARD
