/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// DllHelp.cpp
//
//
// Copyright (c) 2001 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "PreCompiled.h"

#include "TivoApi/DllApi.h"
#include "TivoApi/Dll/DllHelp.h"

#include <string.h>

using namespace TivoApi;


namespace {


//////////////////////////////////////////////////////////////////////////////////////////////////

DllCallbackApi* g_callback;


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Dll
//
// This class is used to implement Tivo_GetApi("DllApi") via the normal registration mechanism.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

class Dll:

    public RefCounted< Singleton<Dll, DllApi> >

{
    public:

        virtual BOOL TIVO_API_METHOD Dll_Initialize(DllCallbackApi* callback)
        {
            g_callback = callback;

            return TRUE;
        }

        ~Dll()
        {
            if (g_callback)
                g_callback->Api_Release();

            g_callback = NULL;
        }

};

Register<Dll, DllApi> DllReg("DllApi");


} // unnamed


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ImplementsBase
//
//////////////////////////////////////////////////////////////////////////////////////////////////

void ImplementsBase::Implements_DtorCalled(Api* old_api)
{
    if (!g_callback)
        return;

    g_callback->Dll_OnApiRelease(old_api);
}


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// RegisterBase
//
//////////////////////////////////////////////////////////////////////////////////////////////////

RegisterBase* RegisterBase::s_back = NULL;


//////////////////////////////////////////////////////////////////////////////////////////////////

RegisterBase::RegisterBase(const char* type_id, int offset, GetApiProc GetApi):
    m_type_id(type_id),
    m_offset(offset),
    m_GetApi(GetApi)
{
    assert(!LookupApi(type_id)); // Oops, your type has already been registered!

    m_next = s_back;
    s_back = this;
}


//////////////////////////////////////////////////////////////////////////////////////////////////

Api* RegisterBase::GetApi(const char* type_id)
{
    RegisterBase* match = LookupApi(type_id);

    if (!match)
        return NULL;

    int new_api = match->m_GetApi();

    if (!new_api)
        return NULL;

    return reinterpret_cast<Api*>(new_api + match->m_offset);
}


//////////////////////////////////////////////////////////////////////////////////////////////////

RegisterBase* RegisterBase::LookupApi(const char* type_id)
{
    for (RegisterBase* iter = s_back; iter; iter = iter->m_next)
    {
        const char* x = iter->m_type_id;

        if (strcmp(x, type_id))
            continue;

        return iter;
    }

    return NULL;
}


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// The entry-point required for every DLL.
//
//////////////////////////////////////////////////////////////////////////////////////////////////

extern "C" Api* WINAPI __export Tivo_GetApi(LPCSTR type_id)
{
    try
    {
        return RegisterBase::GetApi(type_id);
    }

    catch (...)
    {
        return NULL;
    }
}
