/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#ifndef INCLUDE_GUARD_9CCC4869_0EBC_11D5_8018_00A0CCD13742
#define INCLUDE_GUARD_9CCC4869_0EBC_11D5_8018_00A0CCD13742

//////////////////////////////////////////////////////////////////////////////////////////////////
//
// DllHelp.h
//
// SYNOPSIS
//
//  Utility templates for implementing subtypes of Api housed in DLL modules.  Intended for use
//  in conjunction with "ApiHelp.h".
//
//
// EXAMPLE
//
//  Every DLL must implement the standard "Tivo_GetApi" entry-point and use the DllCallbackApi
//  provided during Dll_Initialize().  Linking with this file provides the entry-point for you.
//  However, making it work requires the creation of appropriate instantiations of the Register
//  template throughout your DLL -- which must exist as global or static objects.  Each of your
//  classes must also provide a static GetApi() member function for use by Register.
//
//      using namespace TivoApi;
//
//      class MyFoo: public FooApi
//      {
//          static MyFoo* GetApi() { return new MyFoo(); }
//
//          // Your implementation of FooApi goes here.
//      };
//
//      class MyBar: public BarApi
//      {
//          static MyBar* GetApi() { return new MyBar(); }
//
//          // Your implementation of BarApi goes here.
//      };
//
//      Register<MyFoo, FooApi> FooReg;
//      Register<MyBar, BarApi> BarReg;
//
//
//  Every class that implements an Api should also use the Implements template, which will call
//  your DllCallbackApi automatically at the appropriate times.
//
//      class MyFoo: public Implements<FooApi> { ... };
//
//
//  If you need to support more than one Api in a single class, use one of the "numbered" forms
//  of Implements.
//
//      class MyFooBar: public Implements2<FooApi, BarApi> { ... }
//
//
//  The Implements templates can be combined with other templates from "ApiHelp.h".  Note that
//  the Instanced and Singleton templates implement an appropriate GetApi() member for you.
//
//      class MyFoo: public RefCounted< Instanced< MyFoo, Implements<FooApi> > >
//      {
//          // Your implementation of FooApi goes here -- that's it!
//      };
//
//      Register<MyFoo, FooApi> FooReg;
//
//
// HISTORY
//
//  marnold, 3/8/2001 - Initial version.
//  marnold, 3/9/2001 - Revised comments.
//  marnold, 5/1/2001 - Changed namespace to TivoApi.
//
//
// Copyright (c) 2001 TiVo Inc., all rights reserved.
//
//////////////////////////////////////////////////////////////////////////////////////////////////


#include "TivoApi/ApiHelp.h"

#include <typeinfo>

#include <assert.h>


namespace TivoApi {


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// ImplementsBase
//
// Ignore, this class is an "implementation detail".
//
//////////////////////////////////////////////////////////////////////////////////////////////////

class ImplementsBase
{
    protected:

        void Implements_DtorCalled(Api* old_api);

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Implements, Implements2, Implements3, ...
//
//////////////////////////////////////////////////////////////////////////////////////////////////

template <typename T_Api>
class Implements:

    public T_Api,
    public ImplementsBase

{
    protected:

        ~Implements() { Implements_DtorCalled(this); }

};

template <typename T_Api1, typename T_Api2>
struct Implements2: Implements<T_Api1>, Implements<T_Api2> { };

template <typename T_Api1, typename T_Api2, typename T_Api3>
struct Implements3: Implements<T_Api1>, Implements<T_Api2>, Implements<T_Api3> { };

template <typename T_Api1, typename T_Api2, typename T_Api3, typename T_Api4>
struct Implements4: Implements<T_Api1>, Implements<T_Api2>, Implements<T_Api3>, Implements<T_Api4> { };

template <typename T_Api1, typename T_Api2, typename T_Api3, typename T_Api4, typename T_Api5>
struct Implements5: Implements<T_Api1>, Implements<T_Api2>, Implements<T_Api3>, Implements<T_Api4>, Implements<T_Api5> { };


//////////////////////////////////////////////////////////////////////////////////////////////////

typedef int (*GetApiProc)();


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// RegisterBase
//
// Ignore, this class is an "implementation detail".
//
//////////////////////////////////////////////////////////////////////////////////////////////////

class RegisterBase
{
    public:

        static Api*             GetApi(LPCSTR type_id);


    protected:

        RegisterBase(LPCSTR type_id, int offset, GetApiProc GetApi);

        static RegisterBase*    LookupApi(LPCSTR type_id);

        const LPCSTR            m_type_id;
        const int               m_offset;
        const GetApiProc        m_GetApi;


    private:

        RegisterBase*           m_next;

        static RegisterBase*    s_back;

};


//////////////////////////////////////////////////////////////////////////////////////////////////
//
// Register
//
//////////////////////////////////////////////////////////////////////////////////////////////////

template <typename T, typename T_Api>
class Register:

    private RegisterBase

{
    private:

        static int CalcOffset()
        {
            T* ptr1 = reinterpret_cast<T*>(0xDEADBEEF);
            T_Api* ptr2 = ptr1;
            int val1 = reinterpret_cast<int>(ptr1);
            int val2 = reinterpret_cast<int>(ptr2);

            return val2 - val1;
        }


    public:

        Register(const char* type_id = typeid(T_Api).name()):

            RegisterBase(type_id, CalcOffset(), reinterpret_cast<GetApiProc>(T::GetApi))

        {
        }

};


} // TivoApi


#endif // INCLUDE_GUARD
