/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#include <string.h>
#include <windows.h>
#include <memory>
#include "MpegAudio.h"

///////////////////////////////////////////////////////////////////////////////
template <typename STR_TYPE>
STR_TYPE StrDup(STR_TYPE str)
{
    char *s = new char[strlen(str)+1];
    strcpy(s, str);
    return s;
}
///////////////////////////////////////////////////////////////////////////////
MpegAudio::MpegAudio(LPCSTR filename) :
    m_mp3_full_info(NULL),
    m_mp3_quick_info(NULL),
    m_id3_v1(NULL),
    m_id3_v2(NULL),
    m_filename(StrDup(filename)),
    m_flength(0),
    m_data(NULL),
    m_use_file(true),
    m_cur_seek_ms(0),
    m_cur_seek_offset(0),
    m_min_frames_at_end(0),
    m_last_frame_start_time(0)
{
}
///////////////////////////////////////////////////////////////////////////////
MpegAudio::MpegAudio(DWORD length, const LPCVOID data) :
    m_mp3_full_info(NULL),
    m_mp3_quick_info(NULL),
    m_id3_v1(NULL),
    m_id3_v2(NULL),
    m_filename(NULL),
    m_flength(length),
    m_data(data),
    m_use_file(false),
    m_cur_seek_ms(0),
    m_cur_seek_offset(0),
    m_min_frames_at_end(0),
    m_last_frame_start_time(0)
{
}
///////////////////////////////////////////////////////////////////////////////
MpegAudio::~MpegAudio()
{
    delete [] m_filename;
    delete m_mp3_full_info;
    delete m_mp3_quick_info;
    delete m_id3_v1;
}
///////////////////////////////////////////////////////////////////////////////
BOOL MpegAudio::IsValidMpeg()
{
    if (GetQuickMp3Info())
        return true;

    return false;
}
///////////////////////////////////////////////////////////////////////////////
BOOL MpegAudio::IsId3Present()
{
    if (GetId3v1Info() || GetId3v2Info())
        return true;

    return false;
}
///////////////////////////////////////////////////////////////////////////////
Mp3Info * MpegAudio::GetQuickMp3Info()
{
    if (!m_mp3_quick_info && !m_mp3_full_info)
    {
        bool valid;

        Mp3Info * mp3 = new Mp3Info;

        if (m_use_file)
            valid = GetQuickInfoFromMp3File(m_filename, mp3);
        else
            valid = GetQuickInfoFromMp3Image(m_flength, m_data, mp3);

        if (!valid)
        {
            delete mp3;
            return NULL;
        }

        delete m_mp3_quick_info;

        m_mp3_quick_info = mp3;
    }

    return (m_mp3_quick_info == NULL) ? m_mp3_full_info : m_mp3_quick_info;
}
///////////////////////////////////////////////////////////////////////////////
Mp3Info * MpegAudio::GetFullMp3Info()
{
    if (m_mp3_full_info == NULL)
    {
        m_cur_seek_ms = -1;

        m_mp3_full_info = WalkFullMp3Info();
        m_full_stats = m_stats;
    }

    return m_mp3_full_info;
}
///////////////////////////////////////////////////////////////////////////////
Mp3Info * MpegAudio::WalkFullMp3Info()
{
    bool valid;

    Mp3Info *mp3 = new Mp3Info;

    m_stats.Clear();

    if (m_use_file)
    {
        valid = GetFullInfoFromMp3File
                (
                    mp3,
                    m_filename,
                    this,
                    (Mp3Callback) CallbackThunk
                );
    }
    else
    {
        valid = GetFullInfoFromMp3Image
                (
                    mp3,
                    m_flength,
                    m_data,
                    this,
                    (Mp3Callback)&CallbackThunk
                );
    }
    if (!valid)
    {
        delete mp3;
        return NULL;
    }

    mp3->is_vbr = m_stats.m_is_vbr;

    if (m_stats.m_is_vbr)
    {
        float ave;
        ave = (m_stats.m_vbr_sum_bitrate * 1.0) / m_stats.m_total_frames;
        mp3->vbr_ave_bitrate_bps = (ave * 1000);
    }

    mp3->duration_ms = m_stats.m_total_duration_ms;
    mp3->total_frames = m_stats.m_total_frames;

    // And return it.
    return mp3;
}
///////////////////////////////////////////////////////////////////////////////
id3v1 *   MpegAudio::GetId3v1Info()
{
    if (!m_id3_v1)
    {
        bool valid;

        id3v1* id3 = new id3v1;

        if (m_use_file)
            valid = GetId3v1FromMp3File(m_filename, id3);
        else
            valid = GetId3v1FromMp3Image(m_flength, m_data, id3);

        if (!valid)
        {
            delete id3;
            return NULL;
        }

        m_id3_v1 = id3;
    }

    return m_id3_v1;
}
///////////////////////////////////////////////////////////////////////////////
id3v2 *   MpegAudio::GetId3v2Info()
{
    if (!m_id3_v2)
    {
        bool valid;

        id3v2* id3 = new id3v2;

        if (m_use_file)
            valid = GetId3v2FromMp3File(m_filename, id3);
        else
            valid = GetId3v2FromMp3Image(m_flength, m_data, id3);

        if (!valid)
        {
            delete id3;
            return NULL;
        }

        m_id3_v2 = id3;
    }

    return m_id3_v2;
}
///////////////////////////////////////////////////////////////////////////////
bool MpegAudio::CallbackThunk(void *user, Mp3Info *mp3)
{
    MpegAudio *self = (MpegAudio *) user;

    return self->Callback(mp3);
}
///////////////////////////////////////////////////////////////////////////////
bool MpegAudio::Callback(Mp3Info *mp3)
{
    // Check for vbr
    if (m_stats.m_last_bitrate == -1)
    {
        m_stats.m_last_bitrate = mp3->bitrate_kbps;
    }
    else
    if (mp3->bitrate_kbps != m_stats.m_last_bitrate)
    {
        m_stats.m_is_vbr = true;
    }

    // Add up the bitrates for each frame so we can calc the average later
    m_stats.m_vbr_sum_bitrate += mp3->bitrate_kbps;

    // Add this frame
    m_stats.m_total_frames++;

    // Save this frame's starting time offset
    m_last_frame_start_time = m_stats.m_total_duration_ms;

    // Calc this frame's ending frame time offset
    m_stats.m_total_duration_ms += mp3->frame_duration_ms;

    if (m_cur_seek_ms == -1)
        return true;

    // If the current frame we're processing now is less than the minimum
    // number of frames that the caller always wants to have at the end
    // of the file, then continue processing, looking for the seek point
    if (m_stats.m_total_frames < (m_full_stats.m_total_frames - m_min_frames_at_end))
    {
        // Now check to see if we've hit our target time offset
        if ((long)m_stats.m_total_duration_ms > m_cur_seek_ms)
        {
            m_cur_seek_offset = mp3->frame_offset;
            return false;
        }
    }
    else
    {
        // Otherwise, we'll return this frame here
        m_cur_seek_offset = mp3->frame_offset;
        return false;
    }

    return true;
}
///////////////////////////////////////////////////////////////////////////////
LONG MpegAudio::CalculateOffsetAtTime
(
    DWORD milliseconds,
    int min_frames_at_end,
    int *ms_at_offset
)
{
    std::auto_ptr<Mp3Info> mp3;

    if (milliseconds == 0)
        return 0;

#if 0  // We won't do this for now
    // Quick cache of the last time requested
    if ((milliseconds == (DWORD)m_cur_seek_ms) && min_frames_at_end == m_min_frames_at_end)
        return m_cur_seek_offset;
#endif

    // Make sure that we have the total # of frames in the file
    if (m_full_stats.m_last_bitrate == -1)
    {
        // Looks like we haven't gotten the full # of frames yet,
        // so we'll need to make a full pass through first.

        GetFullMp3Info();
    }

    // First we'll make sure this request is even within the songs playtime
    if (milliseconds > m_full_stats.m_total_duration_ms)
        return -1;

    m_min_frames_at_end = min_frames_at_end;

    m_cur_seek_ms = milliseconds;

    mp3.reset(WalkFullMp3Info());

    if (!mp3.get())
        return -1;


    // If the user wants it, set the time offset for this found frame into
    // his pointer
    if (ms_at_offset)
        *ms_at_offset = m_last_frame_start_time;

    return m_cur_seek_offset;
}
///////////////////////////////////////////////////////////////////////////////
LONG MpegAudio::EstimateOffsetAtTime(DWORD milliseconds)
{
    Mp3Info *mp3;
    LONG off;

    if (milliseconds == 0)
        return 0;

    mp3 = GetQuickMp3Info();

    if (!mp3)
        return -1;

    if (milliseconds >= (DWORD)mp3->duration_ms)
        return -1;

    off = (LONG)(((milliseconds * 1.0) / mp3->duration_ms) * mp3->file_length);

    return off;
}
///////////////////////////////////////////////////////////////////////////////
LONG MpegAudio::EstimateTotalDuration()
{
    if (!GetQuickMp3Info())
        return -1;

    return m_stats.m_total_duration_ms;
}
///////////////////////////////////////////////////////////////////////////////
LONG MpegAudio::CalculateTotalDuration()
{
    if (!GetFullMp3Info())
        return -1;

    return m_full_stats.m_total_duration_ms;
}
///////////////////////////////////////////////////////////////////////////////
