/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
//
// MpegAudio.h
//
// A class for accessing information about Mpeg Audio files
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef INCLUDE_GUARD_4D94BCD3_40EE_49C3_9834_1D270C892FF3
#define INCLUDE_GUARD_4D94BCD3_40EE_49C3_9834_1D270C892FF3

///////////////////////////////////////////////////////////////////////////////
#include <windows.h>
#include "mp3hdr.h"
#include "id3v1.h"
#include "id3v2.h"

///////////////////////////////////////////////////////////////////////////////
struct MpegStats
{
    int             m_last_bitrate;
    DWORD           m_vbr_sum_bitrate;
    DWORD           m_total_frames;
    DWORD           m_total_duration_ms;
    bool            m_is_vbr;

    void Clear()
    {
        m_last_bitrate      = -1;
        m_vbr_sum_bitrate   = 0;
        m_total_frames      = 0;
        m_total_duration_ms = 0;
        m_is_vbr            = false;
    }

    MpegStats()
    {
        Clear();
    }
};
///////////////////////////////////////////////////////////////////////////////
class MpegAudio
{
    protected:

    Mp3Info *       m_mp3_full_info;
    Mp3Info *       m_mp3_quick_info;
    id3v1 *         m_id3_v1;
    id3v2 *         m_id3_v2;

    LPCSTR          m_filename;
    DWORD           m_flength;
    LPCVOID         m_data;
    bool            m_use_file;

    MpegStats       m_stats;
    MpegStats       m_full_stats;

    long            m_cur_seek_ms;
    long            m_cur_seek_offset;

    int             m_min_frames_at_end;
    DWORD           m_last_frame_start_time;

    // Helpers
    static bool     CallbackThunk(void *user, Mp3Info *mp3);

    bool            Callback(Mp3Info *mp3);

    Mp3Info *       WalkFullMp3Info();

    //------------------------------------------------------------------------
    public:

    // Create using the filename of some mpeg audio
    MpegAudio(LPCSTR filename);

    // Create using an in-memory image of some mpeg audio
    MpegAudio(DWORD length, const LPCVOID data);

    // Dtor
    ~MpegAudio();

    // Operations

    // True if this is a valid mpeg audio image
    BOOL      IsValidMpeg();

    // True if there are id3v1 or v2 tags present in the image
    BOOL      IsId3Present();

    // Returns the structure of mp3 info using only a quick frame scan.
    // This means that CBR vs VBR cannot be detected and CBR is assumed and
    // the bitrate_kbps value is calculated from one frame and the length
    // of the image.
    Mp3Info * GetQuickMp3Info();

    // Returns the structure of mp3 info by walking all the frames in the
    // image. This means that both CBR and VBR can be detected and
    // either the bitrate_kbps or vbr_bitrate_kbps value is calculated
    // accurately.
    Mp3Info * GetFullMp3Info();

    // Returns the structure of id3v1 info for this image
    id3v1 *   GetId3v1Info();

    // Returns the structure of id3v2 info for this image
    id3v2 *   GetId3v2Info();

    // Returns the offset into the image that represents the specified number
    // of miliseconds into the song.  Returns -1 for time values that would
    // exceed the duration of the song.  This routine walks every frame up to
    // the requested time value. (slow routine)
    // min_frames_at_end is the minimum number of frames, past which we
    // should never seek.  If the seek point occurs after this minimum
    // number of frames from the end, we'll simply return the offset to the
    // frame after which min_frames_at_end # of valid frames appear before the
    // end of the file.  ms_at_offset can be passed in to get the actual
    // time location for this frame.
    LONG      CalculateOffsetAtTime
              (
                  DWORD milliseconds,
                  int min_frames_at_end = 0,
                  int *ms_at_offset = NULL
              );

    // Returns the offset into the image that represents the specified number
    // of miliseconds into the song.  Returns -1 for time values that would
    // exceed the duration of the song.  This routine only checks the first
    // few frames, assumes that the file is CBR and estimates the offset based
    // on the length of the file and its bitrate. (fast routine)
    LONG      EstimateOffsetAtTime(DWORD milliseconds);


    LONG      EstimateTotalDuration();

    LONG      CalculateTotalDuration();

};

///////////////////////////////////////////////////////////////////////////////
#endif // INCLUDE_GUARD