/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
//
// id3v1.cpp
//
// This provides routines for id3v1 tags
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#ifdef __BORLANDC__
#include <io.h>
#else
#include <sys/stat.h>
#endif

#include "id3v1.h"

///////////////////////////////////////////////////////////////////////////////
static long GetFileLengthFromDescriptor(int fd);
///////////////////////////////////////////////////////////////////////////////
static long GetFileLengthFromDescriptor(int fd)
{
#ifdef __BORLANDC__
    return filelength(fd);
#else
    struct stat s;

    fstat(fd, &s);

    return (long)s.st_size;
#endif
}
///////////////////////////////////////////////////////////////////////////////
static bool GetId3v1FromData(const void *vptr, struct id3v1 *id3)
{
    char *ptr = (char *)vptr;

    if (strncmp(ptr, "TAG", 3) != 0)
        return false;

    ptr+=3;

    memset(id3, 0, sizeof(*id3));

    memcpy(id3->title,ptr,30);
    ptr+=30;
    id3->title[30] = '\0';

    memcpy(id3->artist,ptr,30);
    ptr+=30;
    id3->artist[30] = '\0';

    memcpy(id3->album,ptr,30);
    ptr+=30;
    id3->album[30] = '\0';

    memcpy(id3->year,ptr,4);
    ptr+=4;
    id3->year[4] = '\0';

    memcpy(id3->comment,ptr,30);
    ptr+=30;
    id3->comment[30] = '\0';

    // Check for the id3v1.1 track field
    if(id3->comment[28] == '\0')
        id3->track = id3->comment[29];

    memcpy(&id3->genre,ptr,1);
    // ptr+=1; // Not needed, so this suppresses the assigned without
               // use warning

    id3->genre_string = GetId3GenreStringFromIndex(id3->genre);

    return true;
}
///////////////////////////////////////////////////////////////////////////////
bool GetId3v1FromMp3File(const char *filename, struct id3v1 *id3)
{
    char data[128];
    FILE *fp;

    if ((fp = fopen(filename, "rb")) == NULL)
        return false;

    if (GetFileLengthFromDescriptor(fileno(fp)) < 128)
        return false;

    if (fseek(fp, -128, SEEK_END) != 0)
    {
        fclose(fp);
        return false;
    }

    if (fread(data, 1, 128, fp) != 128)
    {
        fclose(fp);
        return false;
    }

    fclose(fp);

    return GetId3v1FromData(data, id3);
}
///////////////////////////////////////////////////////////////////////////////
bool GetId3v1FromMp3Image(long flength, const void *vptr, struct id3v1 *id3)
{
    char *data = (char *)vptr;

    if (vptr == NULL || flength < 128)
        return false;

    return GetId3v1FromData(&data[flength-128], id3);
}

#ifdef USE_MAIN
///////////////////////////////////////////////////////////////////////////////
int main (int argc, char *argv[])
{
    struct id3v1 id3;

    if (argc < 2)
    {
        printf ("Usage: id3v1.exe mp3filename\n");
        return 0;
    }

#ifdef USE_FILE_IMAGE
    FILE *fp = fopen(argv[1], "rb");

    if (!fp)
    {
        perror(argv[1]);
        return 1;
    }

    unsigned long len = filelength(fileno(fp));

    char *filebuf = new char[len+1];

    fread(filebuf, 1, len, fp);
    fclose(fp);

    if (GetId3v1FromMp3Image(len, filebuf, &id3) == false)
    {
        printf("Error getting id3 info\n");
        return 1;
    }

#else
    if (GetId3v1FromMp3File(argv[1], &id3) == false)
    {
        printf("Error getting id3 info\n");
        return 1;
    }
#endif

    printf("\nfile:    %s\n", argv[1]);
    printf("title:   %s\n", id3.title);
    printf("artist:  %s\n", id3.artist);
    printf("album:   %s\n", id3.album);
    printf("year:    %s\n", id3.year);

    if (id3.comment[0])
        printf("comment: %s\n", id3.comment);

    if (id3.track)
        printf("track:   %d\n", id3.track);

    printf("genre:   %d [%s]\n", id3.genre, id3.genre_string);

    return 0;
}
///////////////////////////////////////////////////////////////////////////////
#endif
