/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

///////////////////////////////////////////////////////////////////////////////
//
// mp3hdr.h
//
// This provides the defines and protos for accessing mp3 frame header info
//
// Copyright (c) 2002 TiVo Inc., all rights reserved.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef INCLUDE_GUARD_5DBA89CA_F900_47CC_900A_2C43981A9A61
#define INCLUDE_GUARD_5DBA89CA_F900_47CC_900A_2C43981A9A61

typedef unsigned char    bits08;
typedef unsigned int     bits32;

///////////////////////////////////////////////////////////////////////////////

enum MpegVersionEnum
{
    mveVersion1_0 = 0,
    mveVersion2_0,
    mveVersion2_5,
};

///////////////////////////////////////////////////////////////////////////////

#pragma option push -a1
struct Mp3FrameHeader
{
    // The first four bytes of the frame are the frame header:

    // The order is LSBit first

    // byte 0
    bits08  start_synch  : 8;

    // byte 1
    bits08  protection   : 1;
    bits08  layer        : 2;
    bits08  id           : 1;
    bits08  end_synch    : 4;

    // byte 2
    bits08  priv         : 1;
    bits08  pad          : 1;
    bits08  sample_rate  : 2;
    bits08  bit_rate     : 4;

    // byte 3
    bits08  emphasis     : 2;
    bits08  original     : 1;
    bits08  copyright    : 1;
    bits08  mode_ext     : 2;
    bits08  channel_mode : 2;

};

#pragma option pop

///////////////////////////////////////////////////////////////////////////////
#pragma option push -a1
struct Mp3Id3v2Header
{
    // The 10 bytes of the id3v2 header

    bits08  signature[3];
    bits08  ver_major;
    bits08  ver_minor;
    bits08  flags;
    bits08  tag_size[4];

#define ID3_V2_SIGNATURE "ID3"
};
#pragma option pop
///////////////////////////////////////////////////////////////////////////////
struct Mp3Info
{
    // This struct can be returned for individual frames, or for the overall
    // file.  Each frame of an mp3 file contains a 4 byte frame header that
    // contains info about that frame, although most of that info is constant
    // across all the frames of the file.  So, (at least in a CBR file) you
    // can examine one frame and find out just about everything you need.

    // Frame specific info
    int               frame_offset;   // Offset into the image of this frame
    int               frame_length;
    int               frame_duration_ms;
    MpegVersionEnum   mpeg_version;
    int               mpeg_layer;
    int               bitrate_kbps;
    int               sample_hz;
    bool              protection;
    bool              padding;
    bool              copyright;
    bool              original;

    // File specific info
    bool              is_vbr;
    long              file_length;
    int               first_frame_offset;  // Offset into the image where the 1st valid frame was found
    int               duration_ms;
    int               total_frames;
    int               vbr_ave_bitrate_bps;

    int               id3v2_tag_offset; // If id3v2 is found, this is the offset
    int               id3v2_length;     // If id3v2 is found, this is its length

    Mp3Info() :
        frame_offset(0),
        frame_length(0),
        frame_duration_ms(0),
        mpeg_version(mveVersion1_0),
        mpeg_layer(0),
        bitrate_kbps(0),
        sample_hz(0),
        protection(false),
        padding(false),
        copyright(false),
        original(false),
        is_vbr(false),
        file_length(0),
        first_frame_offset(0),
        duration_ms(0),
        total_frames(0),
        vbr_ave_bitrate_bps(0.0),
        id3v2_tag_offset(0),
        id3v2_length(0)
    {}

    char *GetMpegVersionString()
    {
        if (mpeg_version == mveVersion1_0)
            return "1.0";
        if (mpeg_version == mveVersion2_0)
            return "2.0";
        if (mpeg_version == mveVersion2_5)
            return "2.5";

        return "";
    }
};

const int consecutive_valid_headers_needed = 3;

typedef bool (*Mp3Callback)(void *user, Mp3Info *mp3);

///////////////////////////////////////////////////////////////////////////////

bool GetQuickInfoFromMp3Image(long flength, const void *vptr, Mp3Info *mp3);

bool GetQuickInfoFromMp3File(const char *fname, Mp3Info *mp3);

bool GetFullInfoFromMp3Image(Mp3Info *pinfo, long flength, const void *vptr, void * user, Mp3Callback callback);

bool GetFullInfoFromMp3File(Mp3Info *pinfo, const char *fname, void * user, Mp3Callback callback);

long CheckImageForId3V2(const void *vptr);

///////////////////////////////////////////////////////////////////////////////

#endif // INCLUDE_GUARD
