/******************************************************************************

    TiVo Desktop Mp3.dll Source Code and Build Process
    Copyright (c) 2003 TiVo Inc.

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or (at
    your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 USA.

    April 4th, 2003

    www.tivo.com

    TiVo Inc.
    P.O. Box 2160
    Alviso, CA 95002

******************************************************************************/

#include <stdio.h>
#include <io.h>
#include "MpegAudio.h"

///////////////////////////////////////////////////////////////////////////////
static char *crack_time(int s)
{
    static char buf[10];

    int hr, min, sec;

    hr = s / (60 * 60);

    s = s - hr * (60 * 60);

    min = s / 60;

    s = s - min * 60;

    sec = s;

    if (hr)
        sprintf(buf, "%02d:%02d:%02d", hr, min, sec);
    else
        sprintf(buf, "%02d:%02d", min, sec);

    return buf;
}
///////////////////////////////////////////////////////////////////////////////
void DumpMp3(Mp3Info *mp3)
{
    printf("size:            %d bytes\n", mp3->file_length);
    if (mp3->id3v2_length)
        printf("ID3 v2 length:   %d bytes\n", mp3->id3v2_length);

    printf("header found at: %d bytes\n", mp3->first_frame_offset);
    printf("Total frames:    %d\n", mp3->total_frames);
    printf("MPEG Version     MPEG %s layer %d\n", mp3->GetMpegVersionString(), mp3->mpeg_layer);
    printf("Copyrighted:     %d\n", mp3->copyright);
    printf("Original:        %d\n", mp3->original);
    printf("CRCs:            %d\n", mp3->protection);
    printf("sample freq:     %dHz\n", mp3->sample_hz);
    if (mp3->is_vbr == false)
    {
        printf("duration:        %s (%d seconds) [%d ms]\n", crack_time(mp3->duration_ms / 1000), mp3->duration_ms / 1000, mp3->duration_ms);
        printf("bitrate:         %dKbps\n", mp3->bitrate_kbps);
    }
    else
    {
        printf("duration:        %s (%d seconds) [%d ms]\n", crack_time(mp3->duration_ms / 1000), mp3->duration_ms / 1000, mp3->duration_ms);
        printf("VBR ave. bitrate %.1fKbps\n", ((float)mp3->vbr_ave_bitrate_bps)/1000.0);
    }

}
///////////////////////////////////////////////////////////////////////////////
#pragma argsused
int main (int argc, char *argv[])
{
    MpegAudio *m;
    Mp3Info *mp3;

    if (argc < 2)
    {
        printf ("Usage: test_mpeg.exe mp3filename\n");
        return 0;
    }

    FILE *fp = fopen(argv[1], "rb");

    if (!fp)
    {
        perror(argv[1]);
        return 1;
    }

    DWORD len = filelength(fileno(fp));

    char *filebuf = new char[len+1];

    fread(filebuf, 1, len, fp);
    fclose(fp);

    m = new MpegAudio(len, filebuf);

    printf("File: %s\n", argv[1]);
    printf("IsValid: %d\n", m->IsValidMpeg());

    printf("\nQuick Info:\n");
    mp3 = m->GetQuickMp3Info();
    if (mp3)
        DumpMp3(mp3);
    else
    {
        printf("Oops, can't get Quick Mp3 info\n");
        goto fini;
    }

    printf("Estimate Offset at 1/2 (%d ms) duration: %d bytes\n",
           mp3->duration_ms/2, m->EstimateOffsetAtTime(mp3->duration_ms/2));

    printf("Estimate Offset at 1ms prior to end (%d): %d bytes\n",
           mp3->duration_ms-1, m->EstimateOffsetAtTime(mp3->duration_ms-1));

    printf("\nFull Info:\n");
    mp3 = m->GetFullMp3Info();
    if (mp3)
        DumpMp3(mp3);
    else
    {
        printf("Oops, can't get full Mp3 info\n");
        goto fini;
    }

    printf("Calculate Offset at 1/2 (%d ms) duration: %d bytes\n",
           mp3->duration_ms/2, m->CalculateOffsetAtTime(mp3->duration_ms/2));

    printf("Actual last frame offset: %d\n", mp3->frame_offset);

    printf("Calculate Offset at 0 ms past total %d bytes\n",
           m->CalculateOffsetAtTime(mp3->duration_ms+0));

    printf("Calculate Offset at 1 ms past total %d bytes\n",
           m->CalculateOffsetAtTime(mp3->duration_ms+1));


    for (int j=0;j<5;j++)
    {
        int real = 0;
        long offset;

        offset = m->CalculateOffsetAtTime(mp3->duration_ms, j, &real);

        printf
        (
            "min: %d, calc offset at end (%d): %d bytes time:%d\n",
            j,
            mp3->duration_ms,
            offset,
            real
        );
    }

    fini: ;

    delete m;

    delete [] filebuf;

    return 0;
}
///////////////////////////////////////////////////////////////////////////////
